/*******************************************************************************
*
*	TITLE:		control.h 
*
*	VERSION:	0.1                           
*
*	DATE:		01-24-2007
*
*	AUTHOR:		Team 1511
*
*	DESCRIPTION:	This file contains controls for the arm, manipulator, and wheels
********************************************************************************
*
*	CHANGE LOG:
*
*	DATE         REV  DESCRIPTION
*	-----------  ---  ----------------------------------------------------------
*	24-Jan-2007  0.1  Curse you Kevin.org!
*
*******************************************************************************/

#ifndef _control_h
#define _control_h

//Control definitions

/*
 * Wrist limits:  - 15 degree to +100 degree from arm projection
 */

// Turret center
#define TURRET_CENTER             113
#define TURRET_CENTER_TOLERANCE   15
#define TURRET_EXTENT             75

// Absolute max positions of all joints
// Extent = 550
#define ARM_MAX     874	//max height of arm before things self destruct
#define ARM_MIN     324

// Extent = 193
#define WRIST_MAX   593
#define WRIST_MIN   400


#define TURRET_MAX (TURRET_CENTER + TURRET_EXTENT)
#define TURRET_MIN (TURRET_CENTER - TURRET_EXTENT)


// Arm too low for turret
#define ARM_MIN_ROTATE (ARM_MIN + 75)

// For calibration, these are the tops of the stingers
// in inches:   104.5  70.6  36.5
// Auto positions for arm
#define ARM_PICKUP      (ARM_MIN + 5)
#define ARM_SCORELO     (ARM_MIN + 147)
#define ARM_SCOREMID    (ARM_MIN + 245)
#define ARM_STRAIGHT    (ARM_MIN + 296) 
#define ARM_SCOREHI     (ARM_MIN + 481)

// Auto positions for wrist
#define WRIST_PICKUP        (WRIST_MIN + 25)
#define WRIST_SCORELO       (WRIST_MIN + 90)
#define WRIST_SCOREMID      (WRIST_MIN + 67)
#define WRIST_ARMSTRAIGHT   (WRIST_MIN + 49)
#define WRIST_SCOREHI        WRIST_MIN

// Wrist position for park
#define WRIST_PARK       WRIST_MAX

// How far can wrist be from target before we stall arm
#define WRIST_DELTA      20

// Maximum amount wrist can be fine adjusted off 
// "optimal" value in auto wrist mode
#define WRIST_FINE_MAX    ((WRIST_MAX - WRIST_MIN) / 2)

// PWM offset to drive when wrist sensor is broken
#define WRIST_MANUAL_UP (127 + 70)
#define WRIST_MANUAL_DOWN (127 - 127)

// Manipulator drive speeds
// CAUTION: If you change these directions, you MUST
// update the function Manip_Bad_From_Drive_Dir in control.c
#define MANIP_DRIVE_OPEN (127 - 40)
#define MANIP_DRIVE_CLOSE (127 + 80)

// Analog value under which gripper pressure sensor
// signals a tube as been grabbed.
#define PRESSURE_THRESHHOLD 950


// Commands for a requested auto arm position
typedef enum
{
  ARM_CMD_NONE,
	ARM_CMD_PARK, 	//go to parked position
	ARM_CMD_PICKUP, //go to pickup position 
	ARM_CMD_LOW, 	//go to low score position
	ARM_CMD_MID, 	//go to mid score position
	ARM_CMD_HI 	//go to high score position
} ArmCommand;


/*******************************************************************************
*		FUNCTION PROTOTYPES
*******************************************************************************/

int Open_Manip(void);
int Close_Manip(void);
void Stop_Manip(void);
// Reads data from manipulator sensors and limit switches to determine state.
// Returns 0 for open, 1 for closed, 2 for grabbed, and 3 for in between.
int Get_Manip_State(void);

/* True if turret in "center". */
int Turret_Centered(void);
int Wrist_At_Limit(void);
int Arm_At_Limit(void);
int Turret_At_Limit(void);

void Arm_Init(void);
void Arm_Reset(void);
void Arm_Sensor_Set_Broken(char turret, char arm, char wrist, char manip);
void Arm_Process(unsigned char arm, int turret, int wrist);

void Arm_Auto_Init(ArmCommand command);
unsigned char Arm_Auto_Process(void);



#endif

