/*******************************************************************************
*
*	TITLE:		motion.h 
*
*	VERSION:	0.1                           
*
*	DATE:		01-25-2007
*
*	AUTHOR:		Team 1511
*
*	DESCRIPTION:	This file contains functions for autonomously driving the wheels.
********************************************************************************
*
*	CHANGE LOG:
*
*	DATE         REV  DESCRIPTION
*	-----------  ---  ----------------------------------------------------------
*	25-Jan-2007  0.1  Basic (working/tested) Ramp_Drive functions.
*
*******************************************************************************/

#ifndef _motion_h
#define _motion_h

/* 2007 robot drive train encoder parameters */
/* 8-inch wheels, 21-inch wheelbase          */
/* So, 66-inch turn circumference            */
/* 200 ticks per revolution                  */

#define TICKS_PER_INCH 8L
#define TICKS_PER_FOOT (TICKS_PER_INCH * 12L)
#define TICKS_PER_NINETY_DEGREES 140L


/*
 * The values below are used for testing with the 2006
 * robot and reflect the difference in # ticks/wheel size
 * These are taken direct from 2006 code and should be
 * reasonably accurate.
 */
/*
#define TICKS_PER_INCH 1L
#define TICKS_PER_FOOT 11L
#define TICKS_PER_NINETY_DEGREES 37L
*/


typedef struct
{
	unsigned char* pwm;
	unsigned char targetSpeed;
	int curDelta;
	unsigned char prevSpeed, prevTarget;
} RampDrive;

#define PID_NO_OSCILLATE -1
typedef struct
{
	long target;
	long tolerance;
	unsigned char* pwm;
	long int_error;
	long prev_error;
  int kp_num, kp_den, ki_num, ki_den, kd_num, kd_den;
  int int_err_max;
  int p_out_max, i_out_max;
  int output_min, output_max;
} PIDData;

/* Initialize this module. Must call this during init! */
void Motion_Init(void);

void Init_RampDrive_Struct(RampDrive* motor, unsigned char* pwm);
void Ramp_Drive(RampDrive* motor);
void Init_PIDData_Struct(PIDData* control, unsigned char* pwm);
void Set_PIDData_Target(PIDData* control, long new_target);
unsigned char PID_Control(PIDData* control, long current);
void Init_Drive_Straight(long ticks);
void Init_Drive_Straight_Limited(long ticks, int max_power);
// MUST set target number of ticks using Init_Drive_Straight
unsigned char Drive_Ticks(void);
// Negative for right, positive for left
void Init_Drive_Turn(long ticks);
// MUST set target number of ticks externally using Init_Drive_Turn()
unsigned char Turn_Ticks(void);

void Auto_Drive (float left_ratio, float right_ratio, int target_speed);
void Reset_Buffers (void);
void Reset_gain_correct (void);
void counter_clockwise_turn (int speed_target);
void clockwise_turn (int speed_target);

#endif // _motion_h
