#include "main.h"

int idMenuSave = wxNewId();
int idMenuQuit = wxNewId();
int idMenuAbout = wxNewId();
int idClear = wxNewId();
int idSave = wxNewId();
int idGenerate = wxNewId();

BEGIN_EVENT_TABLE(MainFrame, wxFrame)
	EVT_MENU(idMenuSave, MainFrame::OnSave)
    EVT_MENU(idMenuQuit, MainFrame::OnQuit)
    EVT_MENU(idMenuAbout, MainFrame::OnAbout)
    EVT_BUTTON(idClear, MainFrame::OnClear)
    EVT_BUTTON(idSave, MainFrame::OnSave)
    EVT_BUTTON(idGenerate, MainFrame::OnGenerate)
END_EVENT_TABLE()

MainFrame::MainFrame(wxFrame *frame, const wxString& title): wxFrame(frame, -1, title)
{
#if wxUSE_MENUS
	wxMenuBar* mbar = new wxMenuBar();
	wxMenu* fileMenu = new wxMenu(_T(""));
	fileMenu->Append(idMenuSave, _("&Save\tCtrl-S"), _("Save the current output"));
	fileMenu->Append(idMenuQuit, _("&Quit\tAlt-F4"), _("Quit the application"));
	mbar->Append(fileMenu, _("&File"));

	wxMenu* helpMenu = new wxMenu(_T(""));
	helpMenu->Append(idMenuAbout, _("&About\tF1"), _("Show info about this application"));
	mbar->Append(helpMenu, _("&Help"));

	SetMenuBar(mbar);
#endif // wxUSE_MENUS

#if wxUSE_STATUSBAR
	CreateStatusBar(2);
	SetStatusText(_(""),1);
	SetStatusText(_("http://www.penfieldrobotics.com"),1);
#endif // wxUSE_STATUSBAR

    label_1 = new wxStaticText(this, -1, _("Array Name:"));
	text_name = new wxTextCtrl(this, -1, _T(""));
	label_2 = new wxStaticText(this, -1, _("Data Type:"));
	const wxString choice_type_choices[] = {
	    _("char"),
	    _("short"),
	    _("long")
	};
	choice_type = new wxChoice(this, -1, wxDefaultPosition, wxDefaultSize, 3, choice_type_choices, 0);
	label_3 = new wxStaticText(this, -1, _("Array Length:"));
	spin_length = new wxSpinCtrl(this, -1, _T("255"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1, MAXARRLEN);
	label_4 = new wxStaticText(this, -1, _("X Min:"));
	spin_xmin = new wxSpinCtrl(this, -1, _T("0"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, XMIN, XMAX);
	label_5 = new wxStaticText(this, -1, _("X Max:"));
	spin_xmax = new wxSpinCtrl(this, -1, _T("255"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, XMIN, XMAX);
	label_6 = new wxStaticText(this, -1, _("f(x):"));
	text_function = new wxTextCtrl(this, -1, _T(""));
	button_clear = new wxButton(this, idClear, _("Clear Output"));
	check_signed = new wxCheckBox(this, -1, _("Signed"));
	button_save = new wxButton(this, idSave, _("Save Output"));
	check_hex = new wxCheckBox(this, -1, _("Hex Output"));
	button_generate = new wxButton(this, idGenerate, _("Generate"));
	check_autoclear = new wxCheckBox(this, -1, _("Autoclear output"));
	text_out = new wxTextCtrl(this, -1, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxHSCROLL);

	set_properties();
	do_layout();
	SetStatusText(_("Generator ready."), 0);
}

MainFrame::~MainFrame()
{
}

void MainFrame::do_layout()
{
	wxBoxSizer* sizer_1 = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer* sizer_2 = new wxBoxSizer(wxHORIZONTAL);
	wxFlexGridSizer* flex_grid_sizer_1 = new wxFlexGridSizer(3, 2, 5, 5);
	flex_grid_sizer_1->SetFlexibleDirection(wxHORIZONTAL);
	flex_grid_sizer_1->SetNonFlexibleGrowMode(wxFLEX_GROWMODE_ALL);
	flex_grid_sizer_1->AddGrowableCol(1, 1);
	wxFlexGridSizer* flex_grid_sizer_2 = new wxFlexGridSizer(3, 2, 5, 5);
	flex_grid_sizer_2->SetFlexibleDirection(wxHORIZONTAL);
	flex_grid_sizer_2->SetNonFlexibleGrowMode(wxFLEX_GROWMODE_ALL);
	flex_grid_sizer_2->AddGrowableCol(1, 1);
	flex_grid_sizer_1->Add(label_1, 0, wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 0);
	flex_grid_sizer_1->Add(text_name, 1, wxEXPAND|wxADJUST_MINSIZE, 0);
	flex_grid_sizer_1->Add(label_2, 0, wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 0);
	flex_grid_sizer_1->Add(choice_type, 1, wxEXPAND|wxADJUST_MINSIZE, 0);
	flex_grid_sizer_1->Add(label_3, 0, wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 0);
	flex_grid_sizer_1->Add(spin_length, 1, wxEXPAND|wxADJUST_MINSIZE, 0);
	flex_grid_sizer_2->Add(label_4, 0, wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 0);
	flex_grid_sizer_2->Add(spin_xmin, 1, wxEXPAND|wxADJUST_MINSIZE, 0);
	flex_grid_sizer_2->Add(label_5, 0, wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 0);
	flex_grid_sizer_2->Add(spin_xmax, 1, wxEXPAND|wxADJUST_MINSIZE, 0);
	flex_grid_sizer_2->Add(label_6, 0, wxALIGN_CENTER_VERTICAL|wxADJUST_MINSIZE, 0);
	flex_grid_sizer_2->Add(text_function, 1, wxEXPAND|wxADJUST_MINSIZE, 0);
	wxGridSizer* grid_sizer_1 = new wxGridSizer(3, 2, 5, 5);
	grid_sizer_1->Add(button_clear, 0, wxALIGN_CENTER_HORIZONTAL|wxADJUST_MINSIZE, 0);
	grid_sizer_1->Add(check_signed, 1, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxADJUST_MINSIZE, 0);
	grid_sizer_1->Add(button_save, 0, wxALIGN_CENTER_HORIZONTAL|wxADJUST_MINSIZE, 0);
	grid_sizer_1->Add(check_hex, 1, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxADJUST_MINSIZE, 0);
	grid_sizer_1->Add(button_generate, 0, wxALIGN_CENTER_HORIZONTAL|wxADJUST_MINSIZE, 0);
	grid_sizer_1->Add(check_autoclear, 1, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxADJUST_MINSIZE, 0);
	sizer_2->Add(flex_grid_sizer_1, 1, wxALL|wxEXPAND, 5);
	sizer_2->Add(flex_grid_sizer_2, 1, wxALL|wxEXPAND, 5);
	sizer_1->Add(sizer_2, 0, wxEXPAND, 0);
	sizer_1->Add(grid_sizer_1, 0, wxEXPAND|wxALIGN_CENTER_HORIZONTAL, 0);
	sizer_1->Add(text_out, 1, wxEXPAND|wxADJUST_MINSIZE, 0);
	SetAutoLayout(true);
	SetSizer(sizer_1);
	sizer_1->Fit(this);
	sizer_1->SetSizeHints(this);
	Layout();
}

void MainFrame::set_properties()
{
	SetTitle(_("Team 1511 Lookup Table Generator"));
	choice_type->SetSelection(0);
	check_autoclear->SetValue(true);
}

void MainFrame::OnQuit(wxCommandEvent& event)
{
    Close();
}

void MainFrame::OnAbout(wxCommandEvent& event)
{
    //wxString msg = wxbuildinfo(long_f);
    //wxMessageBox(msg, _("Welcome to..."));
    AboutFrame* frame = new AboutFrame(0L, _("About..."));
    frame->Show();
}

void MainFrame::OnClear(wxCommandEvent& event)
{
	text_out->SetValue(_T(""));
}

void MainFrame::OnSave(wxCommandEvent& event)
{
	wxFileDialog *save_dialog = new wxFileDialog(this, _("Save File As..."), _(""), _(""),
			_("Text files (*.txt)|*.txt|C Source files (*.c)|*.c| C header files (*.h)|*.h"),
			wxSAVE | wxOVERWRITE_PROMPT, wxDefaultPosition);
	if (save_dialog->ShowModal() == wxID_OK)
	{
		wxString filepath = save_dialog->GetPath();
		text_out->SaveFile(filepath);
	}
}

void MainFrame::OnGenerate(wxCommandEvent& event)
{
	if (text_name->GetValue().length() == 0)
    	wxMessageBox(_("Array name not specified!"), _("Error!"), wxICON_ERROR|wxOK);
	else if (text_function->GetValue().length() == 0)
    	wxMessageBox(_("Function not specified!"), _("Error!"), wxICON_ERROR|wxOK);
	else
	{
		SetStatusText(_("Generating..."), 0);
		FunctionParser fparser;
		fparser.AddConstant("pi", 3.1415926535897932384626433832795);
		std::string function = "";
		function += text_function->GetValue().mb_str();
		std::cout << "f(x) = " << function << '\n';

		int res = fparser.Parse(function, "x");

		if (res >= 0)
		{
			wxString error = _("Error in function at character ");
			error << res;
			error += _T('\n');
			error += wxString(fparser.ErrorMsg(), wxConvUTF8);
			wxMessageBox(error, _("Error!"), wxICON_ERROR|wxOK);

			for(int i = 0; i < res+7; ++i) std::cout << " ";
        	std::cout << "^\n" << fparser.ErrorMsg() << "\n\n";
		}
		else
		{
			// Arrays for limits based on data type
			static const long max_signed[] = {127, 32767, 2147483647};
			static const long max_unsigned[] = {255, 65535, 4294967295};
			static const std::string datatypes[] = {"char ", "short ", "long "};

			std::stringstream output_buf;
			output_buf << "static const rom ";

			if (! check_signed->IsChecked())
				output_buf << "unsigned ";

			output_buf << datatypes[choice_type->GetCurrentSelection()];

			output_buf << text_name->GetValue().mb_str();
			output_buf << " [] = {\n\t";

			double step = (spin_xmax->GetValue() - spin_xmin->GetValue() + 1) / spin_length->GetValue();

			long max;
			if (check_signed->IsChecked())
				max = max_signed[choice_type->GetCurrentSelection()];
			else
				max = max_unsigned[choice_type->GetCurrentSelection()];
			long min = (check_signed->IsChecked() ? -max : 0);

			//Optimize to prepare for calculations
			fparser.Optimize();

			int count = 0;
			double x[1];
			for (x[0] = spin_xmin->GetValue(); x[0] < (spin_xmax->GetValue() +  step); x[0] += step)
			{
					// Get value and round
					long result = lround(fparser.Eval(x));

					// Offset by half-value for signed hex
					if (check_signed->IsChecked() && check_hex->IsChecked())
						result += max_signed[choice_type->GetCurrentSelection()];

					// Send value into stream
					if (check_hex->IsChecked())
					{
						// Keep within hex bounds
						if (result < 0)
							result = 0;
						else if (result > max_unsigned[choice_type->GetCurrentSelection()])
							result = max_unsigned[choice_type->GetCurrentSelection()];

						// Convert to hex with a stream, then the stream to a string
						output_buf << "0x";
						std::stringstream hexstring_stream;
						hexstring_stream << std::hex << result;
						std::string hexstring = hexstring_stream.str();

						// Add spacer 0s to keep tidiness
						static const int hexlen[] = {2, 4, 8};
						for (int hexdigits = hexstring.length();
								hexdigits < hexlen[choice_type->GetCurrentSelection()]; ++hexdigits)
							output_buf << '0';

						output_buf << hexstring;
					}
					else
					{
						// Keep within bounds
						if (result < min)
							result = min;
						else if (result > max)
							result = max;

						output_buf << result;
					}

					if (x[0] < spin_xmax->GetValue())
					{
						output_buf << ", ";
						if ((count + 1) % 8 == 0)
							output_buf << "\n\t";
					}

					++count;
			}
			output_buf << "\n};\n";

			std::cout << output_buf.str();
			if (check_autoclear->IsChecked())
				text_out->SetValue(wxString(output_buf.str().c_str(), wxConvUTF8));
			else
				text_out->SetValue(text_out->GetValue() << wxString(output_buf.str().c_str(), wxConvUTF8));

			SetStatusText(_("Array generated.  Ready to generate."), 0);
		}
	}
}
